package com.zskk.util;

import javax.sql.DataSource;
import com.jfinal.kit.PathKit;
import com.jfinal.kit.PropKit;
import com.jfinal.plugin.activerecord.dialect.MysqlDialect;
import com.jfinal.plugin.activerecord.generator.Generator;
import com.jfinal.plugin.druid.DruidPlugin;

/**
 * 本 demo 仅表达最为粗浅的 jfinal 用法，更为有价值的实用的企业级用法
 * 详见 JFinal 俱乐部: http://jfinal.com/club
 * 
 * 在数据库表有任何变动时，运行一下 main 方法，极速响应变化进行代码重构
 */
public class _JFinalDemoGenerator {
	
	public static DataSource getDataSource() {
		PropKit.use("config.properties");
		DruidPlugin druidPlugin = new DruidPlugin("jdbc:mysql://117.50.71.166:7001/pacsonline_pro?characterEncoding=utf8&zeroDateTimeBehavior=convertToNull", PropKit.get("user"), PropKit.get("password").trim());
		druidPlugin.start();
		return druidPlugin.getDataSource();
	}
	
	public static void main(String[] args) {
		// base model 所使用的包名
		String baseModelPackageName = "com.zskk.model.base";
		// base model 文件保存路径
		String baseModelOutputDir = PathKit.getWebRootPath() + "/src/main/java/com/zskk/model/base";
		
		// model 所使用的包名 (MappingKit 默认使用的包名)
		String modelPackageName = "com.zskk.model";
		// model 文件保存路径 (MappingKit 与 DataDictionary 文件默认保存路径)
		String modelOutputDir = baseModelOutputDir + "/..";
		
		// 创建生成器
		Generator generator = new Generator(getDataSource(), baseModelPackageName, baseModelOutputDir, modelPackageName, modelOutputDir);
		
		// 配置是否生成备注
		generator.setGenerateRemarks(false);
		
		// 设置数据库方言
		generator.setDialect(new MysqlDialect());
		
		// 设置是否生成链式 setter 方法
		generator.setGenerateChainSetter(false);
		
		// 添加不需要生成的表名
		generator.addExcludedTable("bbs","sys_logs","bodypart","constant","department","device","doctor_class","dr_cla_permission","exam_class","exam_project","exam_subclass","images","manager","menu","templates","studies","series","oss_callbacks","version","sys_menus","api","default_cost","every_studies","fa_admin","fa_admin_log","fa_area","fa_attachment","fa_auth_group","fa_auth_group_access","fa_auth_institution_access","fa_auth_rule","fa_category","fa_config","fa_ems","fa_sms","fa_test","fa_user","fa_user_group","fa_user_money_log","fa_user_rule","fa_user_score_log","fa_user_token","fa_version","lcoss_callbacks","log","patient_exam","register","remote_contact","remote_cost","remote_order","remote_water","remote_water_log","send_message","special_contact","special_remote","wechat_pay_log","wechat_refund_log","xz","user_bind","exams","messages","open_application","operating","patient_infos","remote_application","report","report_record","user_wechat","wechat_bind","apis");
		
		// 设置是否在 Model 中生成 dao 对象
		generator.setGenerateDaoInModel(true);
		
		// 设置是否生成字典文件
		generator.setGenerateDataDictionary(false);
		
		// 设置需要被移除的表名前缀用于生成modelName。例如表名 "osc_user"，移除前缀 "osc_"后生成的model名为 "User"而非 OscUser
		generator.setRemovedTableNamePrefixes("t_");
		
		// 生成
		generator.generate();
	}
}




